
# q-expansion of newform 1092.2.bd.b, downloaded from the LMFDB on 22 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1092
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[547, [1, 0]], [365, [-1, 0]], [157, [-1, 0]], [925, [0, 1]]]
aps_data = [[0, 0], [1, 1], [0, 0], [-1, -2], [0, 0], [4, -1], [0, 0], [0, 8], [0, 0], [0, 0], [11, 0], [-4, -4], [0, 0], [0, 5], [0, 0], [0, 0], [0, 0], [18, -9], [-9, -9], [0, 0], [-7, 0], [-17, 0], [0, 0], [0, 0], [0, -19], [0, 0], [-11, 22], [0, 0], [7, -14], [0, 0], [-19, 19], [0, 0], [0, 0], [6, -3], [0, 0], [14, -28], [1, -2], [22, -11], [0, 0], [0, 0], [0, 0], [4, -8], [0, 0], [-7, -7], [0, 0], [30, -15], [-29, 29], [-28, 28], [0, 0], [-22, 0], [0, 0], [0, 0], [0, -14], [0, 0], [0, 0], [0, 0], [0, 0], [-1, 1], [0, -26], [0, 0], [-19, -19], [0, 0], [-19, 0], [0, 0], [-3, 6], [0, 0], [-42, 21], [5, 0], [0, 0], [37, -37], [0, 0], [0, 0], [-9, -9], [0, -25], [15, 15], [0, 0], [0, 0], [0, 35], [0, 0], [0, -7], [0, 0], [-1, 2], [0, 0], [22, -11], [-5, -5], [0, 0], [0, 0], [17, 17], [0, 0], [-21, 42], [0, 0], [0, 0], [4, -2], [0, 0], [-18, 36], [0, 0], [0, 0], [0, 0], [-26, -26], [21, -42], [-1, 0], [0, 0], [0, 0], [0, 0], [16, 0], [46, 0], [0, 0], [0, 0], [0, 0], [24, 24], [52, -26], [9, 9], [0, 0], [49, 0], [30, -15], [0, 0], [0, 7], [0, 0], [0, 0], [0, 0], [-11, 11], [-37, 37], [0, 0], [0, 0], [-49, 49], [0, 0], [-50, 25], [0, 0], [31, -62], [-43, 0], [-30, -30], [0, 0], [-52, 52], [-26, 26], [0, 0], [-2, 2], [0, 0], [0, -31], [0, 0], [0, 0], [-37, 0], [0, 0], [0, 52], [0, 0], [-33, -33], [0, 0], [-23, 0], [0, 0], [-23, 46], [0, 0], [-56, 28], [0, 0], [47, 0], [0, 0], [40, -40], [0, 0], [0, 52], [0, 0], [-32, 64], [0, 0], [0, 0], [0, 0], [34, -68], [0, 0], [0, 0], [0, 0], [44, -44], [46, -23], [35, -70], [0, 0], [0, 0], [0, 14], [0, 0], [-42, 21], [22, -44], [0, 0], [-29, -29], [0, 0], [-31, -31], [25, 25], [38, -76], [0, 0], [43, 0], [0, 0], [0, 0], [0, 0], [19, 0], [32, 0], [3, 3], [0, 0], [-46, 23], [0, 0], [-53, 0], [0, 0], [0, 0], [0, 0], [-19, 38], [0, -67], [0, 0], [0, 0], [0, 0], [58, -29], [-29, 29], [-27, 54], [0, 0], [0, 0], [43, 0], [0, 0], [0, 0], [52, -26], [71, -71], [0, 0], [55, -55], [0, 0], [0, 0], [22, 0], [-61, 0], [0, 0], [0, 0], [0, 0], [-43, 43], [7, 0], [0, 0], [-62, 31], [0, 0], [-71, 0], [0, 0], [0, 0], [74, -37], [0, 0], [-67, 67], [43, 43], [-6, -6], [0, 0], [-1, 2], [0, 0], [0, 43], [0, 0], [0, 0], [0, 0], [0, 0], [26, 26], [-68, 34], [-17, -17], [0, 0], [0, 0], [0, 35], [0, 0], [-10, 5], [0, 0], [7, 7], [0, 0], [0, 0], [45, 45], [0, 0], [0, 0], [0, 26], [80, -80], [0, 0], [0, 70], [-68, 0], [0, 0], [67, 0], [-43, -43], [0, 0], [34, -17], [0, 0], [0, 0], [-84, 42], [0, 0], [5, -10], [33, -66], [10, -10], [0, -52], [-34, 17], [-28, 28], [0, 0], [-12, 24], [50, -25], [0, 0], [0, 0], [30, 30], [0, 0], [-49, 49], [-56, 0], [0, 0], [17, 0], [0, 0], [0, 73], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, -23], [0, 0], [-26, 52], [0, 0], [0, 0], [89, -89], [38, -19], [0, 0], [52, -52]]
